/*
 * Copyright Object Matrix 2015
 */

package com.om.mxs.client.japi.samples;

import java.io.IOException;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;

import com.om.mxs.client.japi.MatrixStore;
import com.om.mxs.client.japi.MatrixStoreConnection;
import com.om.mxs.client.japi.Vault;
import com.om.mxs.client.japi.audit.AuditEntry;
import com.om.mxs.client.japi.audit.AuditEntryDescription;
import com.om.mxs.client.japi.audit.AuditSearchTerm;
import com.om.mxs.client.japi.cred.Credentials;

/**
 * Sample to show how to use auditing functionality. Please note all dates are treated as number of milliseconds since Unix epoch (midnight Jan 1st, 1970)
 * according to cluster time. Having cluster's time synchronised with external time source using NTP protocol is essential.
 * 
 * Search term used to search audits has to contain at least 'from' term created using {@link AuditSearchTerm} createFrom() method.
 */
public class AuditAddSearchSample {

    public static void main(String[] args) throws IOException {
        Vault vault = getVault(args);

        long start = System.currentTimeMillis();
        // build description of the audit entry to be stored
        AuditEntryDescription descr = buildDescr();

        // add entry to a vault
        vault.addAuditEntry(descr);

        // prepare search from-to range
        long from = start - 30 * 1000;
        long to = start + 30 * 1000;

        // simple range query without filtering 
        Iterator<AuditEntry> it = vault.searchAudits(from, to);
        searchWithIterator(it);

        // example of more complicated query
        AuditSearchTerm eq1 = AuditSearchTerm.isEqual("A_SOURCE_IP", "192.168.56.1", false);
        AuditSearchTerm eq2 = AuditSearchTerm.contains("A_APPLICATION", "services", true);
        AuditSearchTerm suffix = AuditSearchTerm.endsWith("A_ENTITY_TYPE", "record", true);

        AuditSearchTerm filter = AuditSearchTerm.and(new AuditSearchTerm[] { eq1, eq2, suffix });

        it = vault.searchAudits(from, filter);
        searchWithIterator(it);
    }

    private static void searchWithIterator(Iterator<AuditEntry> it) {
        while (it.hasNext()) {
            AuditEntry ae = it.next();
            System.out.println("Got entry: " + ae + " with id: " + ae.getUid() + ", for timestamp: " + ae.getTimestamp());
            Map<String, Object> props = ae.getProps();
            System.out.println("Properties: " + props);
        }
    }

    private static AuditEntryDescription buildDescr() {
        long now = System.currentTimeMillis();
        com.om.mxs.client.japi.audit.AuditEntryDescription.Builder b = AuditEntryDescription.newBuilder();
        b.add("A_NOW", Long.toString(now));
        b.add("A_COUNT", Integer.toString(1));
        b.add("A_TIMESTAMP", new Date(now).toString());
        b.add("A_INTERNAL_ID", "fc79a48d-dc58-11e3-9d76-c296976d94dd-225");
        b.add("A_USER_ID", "50ad5ae52b862c9e99fdbe09");
        b.add("A_EVENT_TYPE", "CREATED");
        b.add("A_APPLICATION", "VAULT_SERVICES_SERVER");
        b.add("A_SOURCE_IP", "192.168.56.1");
        b.add("A_FORWARDED_FOR", "192.168.56.11,192.168.56.12");
        b.add("A_ENTITY_ID", "750e78fb-9d20-44e7-a7fa-48fc3c35b836");
        b.add("A_ENTITY_TYPE", "AUDIO_RECORD");
        b.add("A_CASE_ID", "50e64f01-60e2-469b-966c-29adcd0786b1");
        b.add("A_PEERING_ID", "d7ad9bbb-e7eb-4580-9ebf-08b3c098456b");
        b.add("A_DEVICE_ID", "d7ad9bbb-e7eb-4580-9ebf-08b3c098456b");
        return b.build();
    }

    private static Vault getVault(String[] args) throws IOException {
        if (args.length != 2) {
            System.out.println("Invalid number of arguments: " + args.length + ". Values: " + Arrays.toString(args));
            usage(null);
            System.exit(-1);
        }

        // Initialise the variables required to run this example
        // taking the values from the argument line

        // Cluster identifier
        String clusterId = args[0];
        // Comma separated list of IP addresses
        String address = args[1];

        // Reference to vault connection
        Vault vault = null;

        // Create a connection to a MatrixStore vault.
        String[] addresses = address.split(",");
    	MatrixStoreConnection conn = MatrixStoreConnection.builder().withHosts(addresses).withClusterId(clusterId).build();
    	Credentials creds = Credentials.newUsernamePasswordCredentials("testuser", "password");
    	MatrixStore mxs = MatrixStore.builder().withConnection(conn).withCredentials(creds).build();
        vault = mxs.openVault("7c0e47e4-bde2-11e9-a975-cd0d9f8facdc"); 

        // check it's online
        if (!vault.online()) {
            throw new IOException("Vault on accessible");
        }
        return vault;
    }

    private static void usage(String msg) {
        if (msg != null) {
            System.err.println(msg);
        }
        System.err.println("Usage: java AuditAddSearchSample clusterId address");
    }

}
