package com.om.mxs.client.japi.samples;

/**
 * ComplexSearch.java
 *
 * Version 4.1.0.18
 *
 * Copyright (c) 2003-2014 Object Matrix, Ltd. All rights reserved.
 * 
 * Redistribution and use of MatrixStore client API software in source and
 * binary forms ("Software"), with or without modification,
 * is permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright statements
 * and notices, this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * 3. The name "MatrixStore" and "Object Matrix Limited" must not be used to endorse
 * or promote products derived from this Software without prior written permission of
 * Object Matrix Ltd. For written permission, please contact info@object-matrix.com
 * 
 * 4. Products derived from this Software may only utilise this Software to connect to and to 
 * communicate with servers running MatrixStore server software from Object Matrix Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED BY OBJECT MATRIX AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OBJECT MATRIX OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */


import java.io.IOException;
import java.util.Iterator;

import com.om.mxs.client.japi.Attribute;
import com.om.mxs.client.japi.MatrixStore;
import com.om.mxs.client.japi.MatrixStoreConnection;
import com.om.mxs.client.japi.MxsObject;
import com.om.mxs.client.japi.ObjectTypedAttributeView;
import com.om.mxs.client.japi.SearchTerm;
import com.om.mxs.client.japi.Vault;
import com.om.mxs.client.japi.cred.Credentials;

/**
 * This example shows how to search for objects stored in a MatrixStore server with a
 * more complex search term.
 *
 * This example assumes string key and value although the system allows utf keys and
 * any value.
 *
 * The resultant list of object identifiers (OIDs) returned by MatrixStore can be used
 * to access the objects if desired (see RetrieveObject example).
 *
 * For all MatrixStore concepts not explained here please reference
 * the MatrixStore Java API User Guide and the Java API documentation.
 *
 */
public class ComplexSearch {
    
    public static void main(String[] args){

        if (args.length != 2){

            usage(null);
            System.exit(-1);
        }

        // Initialise the variables required to run this example
        // taking the values from the argument line
        
        // Cluster identifier
        String clusterId= args[0];
        // Comma separated list of IP addresses
        String address = args[1];
        
        // Reference to vault connection
        MatrixStoreConnection conn = null;
    	MatrixStore mxs = null;
        Vault vault = null; 

        try{

            // Create a connection to a MatrixStore vault. 
        	String[] addresses = address.split(",");
        	conn = MatrixStoreConnection.builder().withHosts(addresses).withClusterId(clusterId).build();
        	Credentials creds = Credentials.newUsernamePasswordCredentials("testuser", "password");
        	mxs = MatrixStore.builder().withConnection(conn).withCredentials(creds).build();
            vault = mxs.openVault("7c0e47e4-bde2-11e9-a975-cd0d9f8facdc");            

            // Create a first object with metadata but no data
            MxsObject object1 = vault.createObject();
            // create a view to the metadata
            ObjectTypedAttributeView metadata = object1.getAttributeView();
            // add metadata after creation
            String value0= "Cardiff";
            metadata.writeString("Area", value0);
            int value1 = 1;
            metadata.writeInt("Index", value1);

            // Create a second object with metadata and no data
            Attribute[] attributes = new Attribute[2];
            attributes[0] = new Attribute("Area", "Cardiff");
            attributes[1] = new Attribute("Index", (int)2);
            vault.createObject(attributes);
            
            // Search for the first object
            
            // First create the search term that specifies which metadata
            // we are looking for. 
            SearchTerm[] terms = new SearchTerm[2];
            // A simple search term consists of the key and the value
            terms[0] = SearchTerm.createSimpleTerm("Area", "Cardiff");
            // another way to create a term
            terms[1] = SearchTerm.createSimpleTerm(new Attribute("Index", (int)1));
            // complex term we search for
            SearchTerm term = SearchTerm.createANDTerm(terms);
            
            // Perform the search passing the results into an iterator. In this example
            // we limit the result set to be 100 object identifiers.
            Iterator<String> iterator = vault.searchObjectsIterator(term, 100);            
            
            // Loop through the result iterator printing the results to the console.
            // You could change this section to read back the objects associated with
            // the resultant list of object identifiers.
            int j = 0;
            String id = null;

            System.out.println ("The MatrixStore objects containing both attributes are:");
            
            while (iterator.hasNext()){
                id = (String)iterator.next();
                System.out.println (id);
                j++;
            }

            // No objects matched the search criteria
            if (j==0){
                System.out.println ("Sorry, no objects matched the search criteria");
            }

        }catch (IOException e){
 
            // A MatrixStore error code will be returned along with a message
            // explaining the meaning of the error.
            usage("MatrixStore Error: code=" + Vault.getErrorCode(e) +
                  " message=" + e.getMessage());

        }catch (Exception e){

            usage("Error: " + e.getMessage());
            
        }finally{
            
            // Close all the remaining objects
        	if (vault != null)
                vault.dispose();
            if (mxs != null)
            	mxs.dispose();
            if (conn != null)
            	conn.close();
        }
    }

    private static void usage (String msg){

        if (msg != null){
            System.err.println (msg);
        }
        System.err.println ("Usage: java ComplexSearch clusterId address");
    }    

}
