/*
 * Copyright Object Matrix 2018
 */

package com.om.mxs.client.japi.samples;

import java.io.IOException;
import java.util.Set;

import com.om.mxs.client.japi.EntityType;
import com.om.mxs.client.japi.IdentityManagement;
import com.om.mxs.client.japi.MatrixStore;
import com.om.mxs.client.japi.MatrixStoreConnection;
import com.om.mxs.client.japi.Permissions;
import com.om.mxs.client.japi.cred.Credentials;
import com.om.mxs.client.japi.iam.IdentityPrivileges;
import com.om.mxs.client.japi.iam.Permission;
import com.om.mxs.client.japi.iam.Privilege;
import com.om.mxs.client.japi.iam.UserInformation;

/**
  Example to show how to get user information and permissions once authenticated.
  
  Example output:
    Logged in as: 'API Test User'. User has ID: 5fd056c5-b5b4-11e8-b963-ef69c21c706d
    On type Vault with ID: c85ccb4b-b5b6-11e8-b963-ef69c21c706d user has permissions: 
            [Objects_UpdateMeta, Objects_Read, Objects_Write, Objects_Delete, Objects_Search]
    For vault: c85ccb4b-b5b6-11e8-b963-ef69c21c706d user has permissions: 
            [Objects_UpdateMeta, Objects_Read, Objects_Write, Objects_Delete, Objects_Search]
    User has Objects_Read permission? true
*/
public class IdentityManagementSample {

    private static final String[] hosts = new String[] { "node153", "node154" };
    private static final String vaultId = "c85ccb4b-b5b6-11e8-b963-ef69c21c706d";
    
    public static void main(String[] args) throws IOException {
    
        // prepare credentials used to authenticate user
        String username = "apitest";
        char[] password = "password".toCharArray();
        Credentials cred = Credentials.newUsernamePasswordCredentials(username, password);
        
        MatrixStoreConnection conn = MatrixStoreConnection.builder().withHosts(hosts).build();
        MatrixStore mxs = MatrixStore.builder().withConnection(conn).withCredentials(cred).build();
        
        // once authenticated user can get instance of identity management
        IdentityManagement idm = mxs.getIdentityManagement();

        // user can get information about himself/herself
        retrieveCurrentUserInformation(idm);

        mxs.closeConnection();
    }
    
    private static void retrieveCurrentUserInformation(IdentityManagement idm) throws IOException {
        UserInformation userinfo = idm.currentUserInformation();
        String userid = userinfo.getId();
        String fullName = userinfo.getFullName();
        System.err.println("Logged in as: '" + fullName + "'. User has ID: " + userid);

        // user can change password
        // idm.changePassword("newpassword".toCharArray());
        
        // user can retrieve privileges of an identity
        IdentityPrivileges privileges = idm.currentUserVaultsPrivileges();
        Privilege[] privilegesArray = privileges.getPrivileges();
        
        for (Privilege privilege : privilegesArray) {
            String entityId = privilege.getEntityId();
            EntityType entityType = privilege.getEntityType();
            Set<Permission> permissions = privilege.getPermissions();
            System.err.println("On type " + entityType + " with ID: " + entityId + " user has permissions: " + permissions);
        }
        
        // user can get permissions for specific entity
        Set<Permission> permissions = privileges.permissionsForEntity(vaultId);
        System.err.println("For vault: " + vaultId + " user has permissions: " + permissions);
        
        // user can check if has a specific permission on entity
        Permission perm = Permissions.Read;
        boolean hasPermission = privileges.hasPermissionOn(vaultId, perm);
        System.err.println("User has " + perm + " permission? " + hasPermission);
    }
}
