package com.om.mxs.client.japi.samples;

/**
 * MakeObjectOfflineOnline.java
 *
 * Version 4.1.0.18
 *
 * Copyright (c) 2003-2014 Object Matrix, Ltd. All rights reserved.
 * 
 * Redistribution and use of MatrixStore client API software in source and
 * binary forms ("Software"), with or without modification,
 * is permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright statements
 * and notices, this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * 3. The name "MatrixStore" and "Object Matrix Limited" must not be used to endorse
 * or promote products derived from this Software without prior written permission of
 * Object Matrix Ltd. For written permission, please contact info@object-matrix.com
 * 
 * 4. Products derived from this Software may only utilise this Software to connect to and to 
 * communicate with servers running MatrixStore server software from Object Matrix Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED BY OBJECT MATRIX AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OBJECT MATRIX OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;
import java.nio.channels.ByteChannel;
import java.nio.channels.FileChannel;
import java.util.Date;

import com.om.mxs.client.japi.AccessOption;
import com.om.mxs.client.japi.MXFSFileAttributeView;
import com.om.mxs.client.japi.MXFSFileAttributes;
import com.om.mxs.client.japi.MatrixStore;
import com.om.mxs.client.japi.MatrixStoreConnection;
import com.om.mxs.client.japi.MxsObject;
import com.om.mxs.client.japi.Vault;
import com.om.mxs.client.japi.cred.Credentials;

/**
 * This example shows how to make an object offline and back online in MatrixStore. 
 * 
 * When an object is offline, the object is still searchable and its metadata 
 * is accessible but the data is cleared from the system. 
 * The data is not accessible until the application feeds it back to the object and marks it as online.
 * 
 * An application of this feature is to move an object to an LTO system to free space on MatrixStore
 * and recover the data when is needed.
 *
 */
public class MakeObjectOfflineOnline {


    public static void main(String[] args) throws IOException {

        if (args.length != 3){
            usage(null);
            System.exit(-1);
        }

        // Initialise the variables required to run this example
        // taking the values from the argument line

        // Cluster identifier
        String clusterId= args[0];
        // Comma separated list of IP addresses 
        String address= args[1];
        // Identifier of the object to retrieve
        String objectId= args[2];

        // Reference to vault connection
        MatrixStoreConnection conn = null;
    	MatrixStore mxs = null;
        Vault vault = null; 

        FileOutputStream outData = null;
        try{

            // First, we save the data of the object locally so we can feed it back
            // The data could be saved on another device like an LTO robot.
            
            // Create the directory and the files where  
            // the object's data will be saved
            File objectDir= new File (objectId);
            objectDir.mkdir();

            // Create a local data file to hold the data
            outData= new FileOutputStream (objectDir.getName() + 
                    File.separator+ "data");
            FileChannel outChannel = outData.getChannel();
            
            // Create a connection to a MatrixStore vault. 
            String[] addresses = address.split(",");
        	conn = MatrixStoreConnection.builder().withHosts(addresses).withClusterId(clusterId).build();
        	Credentials creds = Credentials.newUsernamePasswordCredentials("testuser", "password");
        	mxs = MatrixStore.builder().withConnection(conn).withCredentials(creds).build();
            vault = mxs.openVault("7c0e47e4-bde2-11e9-a975-cd0d9f8facdc");           

            // Create an object instance. 
            // null object id means that the object is in memory and not in the server yet
            MxsObject object = vault.getObject(objectId);

            // Open an input channel to the given object identified by objectId.
            // We'll read the complete object so we'll start from offset = 0
            ByteChannel inChannel = object.newObjectChannel(AccessOption.READ);

            // Create a byte buffer to put the retrieved data.
            // A good buffer size for transmissions is 64KB segments.
            ByteBuffer data = ByteBuffer.allocate(64 * 1024);            
            // Read until the end of the data
            while (inChannel.read(data) != -1){
                data.flip();
                outChannel.write(data);
                data.clear();
            }
            // Close the channels used
            inChannel.close();
            outChannel.close();

            // make the object offline specifying the application identifier 
            object.makeOffline("MakeObjectOfflineOnline");
            
            // check you can still access metadata for this object
            MXFSFileAttributeView view = object.getMXFSFileAttributeView();
            MXFSFileAttributes attributes = view.readAttributes();
            if (!object.isOnline()){
                System.out.println("Object is offline");
            }
            // When and object is offline, its size should be zero 
            System.out.println("Object size: " + attributes.size());
            System.out.println("Object creation date: " + new Date(attributes.creationTime().longValue()));
            System.out.println("is MXFS object?: " + !attributes.isOther());
            System.out.println("Who made the object offine?: " + object.getAttributeView().readString("MXS_APPLICATION_ID"));

            // check you cannot access the data
            try{
                inChannel = object.newObjectChannel(AccessOption.READ);
            }
            catch (IOException e){
                System.out.println("MatrixStore Error: code=" + Vault.getErrorCode(e) +
                        " message=" + e.getMessage());
            }
            
            // make the object online, feed the data to the object
            FileInputStream in = new FileInputStream (objectDir.getName() + 
                    File.separator+ "data");
            byte inData[]= new byte[64 * 1024];

            OutputStream outStream = object.makeOnline();

            // Read the data from the file and send it to the server
            int numBytes = 0;
            while ((numBytes = in.read(inData)) != -1){
                outStream.write(inData, 0, numBytes);
            }

            // Close the streams
            in.close();
            outStream.close();

            // check you can now access the data again and no error is thrown
            inChannel = object.newObjectChannel(AccessOption.READ);
            // do nothing
            inChannel.close();
            
            // Object's size is back to it's original
            view = object.getMXFSFileAttributeView();
            attributes = view.readAttributes();
            if (object.isOnline()){
                System.out.println("Object is online");
            }
            System.out.println("Object size: " + attributes.size());

            
        }catch (IOException e){

            // A MatrixStore error code will be returned along with a message
            // explaining the meaning of the error.
            usage("MatrixStore Error: code=" + Vault.getErrorCode(e) +
                    " message=" + e.getMessage());

        }catch (Exception e){

            usage("Error: " + e.getMessage());

        }finally{
        	if (outData != null)
        		outData.close();

            // Close all the remaining instances
        	if (vault != null)
                vault.dispose();
            if (mxs != null)
            	mxs.dispose();
            if (conn != null)
            	conn.close();
        }
    }

    private static void usage (String msg){

        if (msg != null){
            System.err.println (msg);
        }
        System.err.println ("Usage: java MakeObjectOfflineOnline clusterId address objectId");
    }


}
