package com.om.mxs.client.japi.samples;

/**
 * RetrieveObject.java
 *
 * Version 4.1.0.18
 *
 * Copyright (c) 2003-2014 Object Matrix, Ltd. All rights reserved.
 * 
 * Redistribution and use of MatrixStore client API software in source and
 * binary forms ("Software"), with or without modification,
 * is permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright statements
 * and notices, this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * 3. The name "MatrixStore" and "Object Matrix Limited" must not be used to endorse
 * or promote products derived from this Software without prior written permission of
 * Object Matrix Ltd. For written permission, please contact info@object-matrix.com
 * 
 * 4. Products derived from this Software may only utilise this Software to connect to and to 
 * communicate with servers running MatrixStore server software from Object Matrix Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED BY OBJECT MATRIX AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OBJECT MATRIX OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.Iterator;

import com.om.mxs.client.japi.Attribute;
import com.om.mxs.client.japi.MatrixStore;
import com.om.mxs.client.japi.MatrixStoreConnection;
import com.om.mxs.client.japi.MxsObject;
import com.om.mxs.client.japi.ObjectTypedAttributeView;
import com.om.mxs.client.japi.Vault;
import com.om.mxs.client.japi.cred.Credentials;

/**
 * This example shows how to retrieve the data and metadata attributes
 * from an existing MatrixStore object.
 *
 * Those object attributes are saved into a new directory in the current path 
 * called as the object identifier.
 *   
 * For all MatrixStore concepts not explained here please reference
 * the MatrixStore Java API User Guide and the Java API documentation.
 *
 */
public class RetrieveObject {


    public static void main(String[] args) {

        if (args.length != 3){
            usage(null);
            System.exit(-1);
        }

        // Initialise the variables required to run this example
        // taking the values from the argument line

        // Cluster identifier
        String clusterId= args[0];
        // Comma separated list of IP addresses 
        String address= args[1];
        // Identifier of the object to retrieve
        String objectId= args[2];
        
        // Reference to vault connection
        MatrixStoreConnection conn = null;
    	MatrixStore mxs = null;
        Vault vault = null;  
        
        try{
            
            // Create the directory and the files where  
            // the object's data and metadata attributes
            // will be saved
            File objectDir= new File (objectId);
            objectDir.mkdir();
        
            // Create a local data file to hold the data
            FileOutputStream outData= new FileOutputStream (objectDir.getName() + 
                                                            File.separator+ "data");
            // Create a local text file to hold the metadata.
            // We'll assume that the metadata only contain text values
            BufferedWriter outMetadata= new BufferedWriter(
                           new FileWriter (objectDir.getName() +
                                           File.separator + "metadata"));

            // Create a connection to a MatrixStore vault. 
            String[] addresses = address.split(",");
        	conn = MatrixStoreConnection.builder().withHosts(addresses).withClusterId(clusterId).build();
        	Credentials creds = Credentials.newUsernamePasswordCredentials("testuser", "password");
        	mxs = MatrixStore.builder().withConnection(conn).withCredentials(creds).build();
            vault = mxs.openVault("7c0e47e4-bde2-11e9-a975-cd0d9f8facdc");               
            
            // Create an object instance. 
            // null object id means that the object is in memory and not in the server yet
            MxsObject object = vault.getObject(objectId);

            // Open an input stream to the given object identified by objectId.
            // We'll read the complete object so we'll start from offset = 0
            InputStream inStream = object.newInputStream();

            // Create a byte array to put the retrieved data.
            // A good size is 64 KB
            byte[] buffer= new byte [64 * 1024];            
            int chunkSize= buffer.length;
            int numRead=0;
            // We re-fill the buffer on every iteration, so set the buffer
            // offset to zero
            int offset= 0;
            
            // Read until the end of the data
            while ((numRead= inStream.read(buffer, offset, chunkSize)) != -1){
                outData.write(buffer, 0, numRead);
            }

            // Close the objects used
            inStream.close();
            outData.close();

            // create a view to the metadata
            ObjectTypedAttributeView metadata = object.getAttributeView();
            
//            // Get attributes directly
//            String area = metadata.readString("Area");
//            String type = metadata.readString("ImageType");
//            int code = metadata.readInt("Code");
            
            // Get the list of metadata attributes with a number of attributes per network call
            Iterator<Attribute> iterator= metadata.iterator(10);
            while (iterator.hasNext()){
              
                // Get an attribute
                Attribute attr = iterator.next();
                
                // Save the attribute into the metadata file.
                // With the iterator, you need to give the hint of the type of value you expect
                if (attr.getKey().equals("Code")){
                    int value = ((Integer)attr.getValue((int)0)).intValue();
                    outMetadata.write(attr.getKey() + " " + value);
                }else{
                    outMetadata.write(attr.getKey() + " " + attr.getValue(""));
                }
                outMetadata.newLine();
            }
            
            outMetadata.close();
            
        }catch (IOException e){
            
            // A MatrixStore error code will be returned along with a message
            // explaining the meaning of the error.
            usage("MatrixStore Error: code=" + Vault.getErrorCode(e) +
                  " message=" + e.getMessage());
            
        }catch (Exception e){
        
            usage("Error: " + e.getMessage());
            
        }finally{
            
            // Close all the remaining objects
        	if (vault != null)
                vault.dispose();
            if (mxs != null)
            	mxs.dispose();
            if (conn != null)
            	conn.close();
        }
    }

    private static void usage (String msg){
        
        if (msg != null){
            System.err.println (msg);
        }
        System.err.println ("Usage: java RetrieveObject clusterId address objectId");
    }
    
}
