package com.om.mxs.client.japi.samples;

/**
 * RetrieveObjectInformation.java
 *
 * Version 4.1.0.18
 *
 * Copyright (c) 2003-2014 Object Matrix, Ltd. All rights reserved.
 * 
 * Redistribution and use of MatrixStore client API software in source and
 * binary forms ("Software"), with or without modification,
 * is permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright statements
 * and notices, this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 
 * 3. The name "MatrixStore" and "Object Matrix Limited" must not be used to endorse
 * or promote products derived from this Software without prior written permission of
 * Object Matrix Ltd. For written permission, please contact info@object-matrix.com
 * 
 * 4. Products derived from this Software may only utilise this Software to connect to and to 
 * communicate with servers running MatrixStore server software from Object Matrix Ltd.
 * 
 * THIS SOFTWARE IS PROVIDED BY OBJECT MATRIX AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL OBJECT MATRIX OR ITS
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 */

import java.io.IOException;
import java.io.OutputStream;
import java.util.Date;

import com.om.mxs.client.japi.MXFSFileAttributeView;
import com.om.mxs.client.japi.MXFSFileAttributes;
import com.om.mxs.client.japi.MatrixStore;
import com.om.mxs.client.japi.MatrixStoreConnection;
import com.om.mxs.client.japi.MxsObject;
import com.om.mxs.client.japi.Vault;
import com.om.mxs.client.japi.cred.Credentials;

/**
 * This example shows how to retrieve common MatrixStore object information.
 *
 * For all MatrixStore concepts not explained here please reference
 * the MatrixStore Java API User Guide and the Java API documentation.
 *
 */
public class RetrieveObjectInformation {

    public static void main(String[] args){

        if (args.length != 2){

            usage(null);
            System.exit(-1);
        }

        // Initialise the variables required to run this example
        // taking the values from the argument line
        
        // Cluster identifier
        String clusterId= args[0];
        // Comma separated list of IP addresses
        String address = args[1];
        
        // Reference to vault connection
        MatrixStoreConnection conn = null;
    	MatrixStore mxs = null;
        Vault vault = null; 

        try{
            // Create a connection to a MatrixStore vault. 
        	String[] addresses = address.split(",");
        	conn = MatrixStoreConnection.builder().withHosts(addresses).withClusterId(clusterId).build();
        	Credentials creds = Credentials.newUsernamePasswordCredentials("testuser", "password");
        	mxs = MatrixStore.builder().withConnection(conn).withCredentials(creds).build();
            vault = mxs.openVault("7c0e47e4-bde2-11e9-a975-cd0d9f8facdc");             

            // Create an object with some data
            // Open an output stream
            MxsObject object = vault.getObject(null);
            OutputStream outStream= object.newOutputStream();
            byte[] initData = new byte[] {'a', 'a', 'a'};
            outStream.write(initData);
            // Close the streams
            outStream.close();
            
            System.out.println("Object created: " + object.getId());

            // Get MXFS information
            MXFSFileAttributeView view = object.getMXFSFileAttributeView();
            MXFSFileAttributes attributes = view.readAttributes();
            
            System.out.println("Object size: " + attributes.size());
            System.out.println("Object creation date: " + new Date(attributes.creationTime().longValue()));
            System.out.println("is MXFS object?: " + !attributes.isOther());


        }catch (IOException e){
            
            // A MatrixStore error code will be returned along with a message
            // explaining the meaning of the error.
            usage("MatrixStore Error: code=" + Vault.getErrorCode(e) +
                  " message=" + e.getMessage());

        }catch (Exception e){

            usage("Error: " + e.getMessage());
            
        }finally{
            
            // Close all the remaining objects
        	if (vault != null)
                vault.dispose();
            if (mxs != null)
            	mxs.dispose();
            if (conn != null)
            	conn.close();
        }

    }
    
    private static void usage (String msg){

        if (msg != null){
            System.err.println (msg);
        }
        System.err.println ("Usage: java RetrieveObjectInformation clusterId address");
    }    
            
}
